@@ -174,55 +174,55 @@ fn iter_sum_2d_transpose_by_row(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn scalar_sum_2d_regular(bench: &mut test::Bencher)
+fn sum_2d_regular(bench: &mut test::Bencher)
 {
     let a = Array::<i32, _>::zeros((64, 64));
     let a = black_box(a);
     bench.iter(|| {
-        a.scalar_sum()
+        a.sum()
     });
 }
 
 #[bench]
-fn scalar_sum_2d_cutout(bench: &mut test::Bencher)
+fn sum_2d_cutout(bench: &mut test::Bencher)
 {
     let a = Array::<i32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
-        a.scalar_sum()
+        a.sum()
     });
 }
 
 #[bench]
-fn scalar_sum_2d_float(bench: &mut test::Bencher)
+fn sum_2d_float(bench: &mut test::Bencher)
 {
     let a = Array::<f32, _>::zeros((64, 64));
     let a = black_box(a.view());
     bench.iter(|| {
-        a.scalar_sum()
+        a.sum()
     });
 }
 
 #[bench]
-fn scalar_sum_2d_float_cutout(bench: &mut test::Bencher)
+fn sum_2d_float_cutout(bench: &mut test::Bencher)
 {
     let a = Array::<f32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]);
     let a = black_box(av);
     bench.iter(|| {
-        a.scalar_sum()
+        a.sum()
     });
 }
 
 #[bench]
-fn scalar_sum_2d_float_t_cutout(bench: &mut test::Bencher)
+fn sum_2d_float_t_cutout(bench: &mut test::Bencher)
 {
     let a = Array::<f32, _>::zeros((66, 66));
     let av = a.slice(s![1..-1, 1..-1]).reversed_axes();
     let a = black_box(av);
     bench.iter(|| {
-        a.scalar_sum()
+        a.sum()
     });
 }
 
