@@ -71,7 +71,8 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     type Smaller: Dimension;
     /// Next larger dimension
     type Larger: Dimension;
-    #[doc(hidden)]
+
+    /// Returns the number of dimensions (number of axes).
     fn ndim(&self) -> usize;
 
     /// Convert the dimension into a pattern matching friendly value.
@@ -108,12 +109,17 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
         self.slice() == rhs.slice()
     }
 
+    /// Returns the strides for a standard layout array with the given shape.
+    ///
+    /// If the array is non-empty, the strides result in contiguous layout; if
+    /// the array is empty, the strides are all zeros.
     #[doc(hidden)]
     fn default_strides(&self) -> Self {
         // Compute default array strides
         // Shape (a, b, c) => Give strides (b * c, c, 1)
-        let mut strides = self.clone();
-        {
+        let mut strides = Self::zeros(self.ndim());
+        // For empty arrays, use all zero strides.
+        if self.slice().iter().all(|&d| d != 0) {
             let mut it = strides.slice_mut().iter_mut().rev();
             // Set first element to 1
             while let Some(rs) = it.next() {
@@ -129,12 +135,17 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
         strides
     }
 
+    /// Returns the strides for a Fortran layout array with the given shape.
+    ///
+    /// If the array is non-empty, the strides result in contiguous layout; if
+    /// the array is empty, the strides are all zeros.
     #[doc(hidden)]
     fn fortran_strides(&self) -> Self {
         // Compute fortran array strides
         // Shape (a, b, c) => Give strides (1, a, a * b)
-        let mut strides = self.clone();
-        {
+        let mut strides = Self::zeros(self.ndim());
+        // For empty arrays, use all zero strides.
+        if self.slice().iter().all(|&d| d != 0) {
             let mut it = strides.slice_mut().iter_mut();
             // Set first element to 1
             while let Some(rs) = it.next() {
@@ -150,20 +161,13 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
         strides
     }
 
-    #[doc(hidden)]
-    // Return an index of same dimensionality
-    fn zero_index(&self) -> Self {
-        Self::default()
-    }
-
-    #[doc(hidden)]
-    /// Return an index of same type and with the specified dimensionality.
+    /// Creates a dimension of all zeros with the specified ndim.
     ///
     /// This method is useful for generalizing over fixed-size and
     /// variable-size dimension representations.
     ///
     /// **Panics** if `Self` has a fixed size that is not `ndim`.
-    fn zero_index_with_ndim(ndim: usize) -> Self;
+    fn zeros(ndim: usize) -> Self;
 
     #[doc(hidden)]
     #[inline]
@@ -173,11 +177,7 @@ pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
                 return None;
             }
         }
-        let mut index = self.clone();
-        for rr in index.slice_mut().iter_mut() {
-            *rr = 0;
-        }
-        Some(index)
+        Some(Self::zeros(self.ndim()))
     }
 
     #[doc(hidden)]
@@ -380,7 +380,7 @@ impl Dimension for Dim<[Ix; 0]> {
     #[inline]
     fn into_pattern(self) -> Self::Pattern { }
     #[inline]
-    fn zero_index_with_ndim(ndim: usize) -> Self {
+    fn zeros(ndim: usize) -> Self {
         assert_eq!(ndim, 0);
         Self::default()
     }
@@ -416,7 +416,7 @@ impl Dimension for Dim<[Ix; 1]> {
         get!(&self, 0)
     }
     #[inline]
-    fn zero_index_with_ndim(ndim: usize) -> Self {
+    fn zeros(ndim: usize) -> Self {
         assert_eq!(ndim, 1);
         Self::default()
     }
@@ -442,7 +442,11 @@ impl Dimension for Dim<[Ix; 1]> {
 
     #[inline]
     fn default_strides(&self) -> Self {
-        Ix1(1)
+        if get!(self, 0) == 0 {
+            Ix1(0)
+        } else {
+            Ix1(1)
+        }
     }
 
     #[inline]
@@ -510,7 +514,7 @@ impl Dimension for Dim<[Ix; 2]> {
     #[inline]
     fn slice_mut(&mut self) -> &mut [Ix] { self.ixm() }
     #[inline]
-    fn zero_index_with_ndim(ndim: usize) -> Self {
+    fn zeros(ndim: usize) -> Self {
         assert_eq!(ndim, 2);
         Self::default()
     }
@@ -558,13 +562,23 @@ impl Dimension for Dim<[Ix; 2]> {
 
     #[inline]
     fn default_strides(&self) -> Self {
-        // Compute default array strides
-        // Shape (a, b, c) => Give strides (b * c, c, 1)
-        Ix2(get!(self, 1), 1)
+        let m = get!(self, 0);
+        let n = get!(self, 1);
+        if m == 0 || n == 0 {
+            Ix2(0, 0)
+        } else {
+            Ix2(n, 1)
+        }
     }
     #[inline]
     fn fortran_strides(&self) -> Self {
-        Ix2(1, get!(self, 0))
+        let m = get!(self, 0);
+        let n = get!(self, 1);
+        if m == 0 || n == 0 {
+            Ix2(0, 0)
+        } else {
+            Ix2(1, m)
+        }
     }
 
     #[inline]
@@ -655,7 +669,7 @@ impl Dimension for Dim<[Ix; 3]> {
     }
 
     #[inline]
-    fn zero_index_with_ndim(ndim: usize) -> Self {
+    fn zeros(ndim: usize) -> Self {
         assert_eq!(ndim, 3);
         Self::default()
     }
@@ -764,7 +778,7 @@ macro_rules! large_dim {
             #[inline]
             fn slice_mut(&mut self) -> &mut [Ix] { self.ixm() }
             #[inline]
-            fn zero_index_with_ndim(ndim: usize) -> Self {
+            fn zeros(ndim: usize) -> Self {
                 assert_eq!(ndim, $n);
                 Self::default()
             }
@@ -817,12 +831,7 @@ impl Dimension for IxDyn
     }
 
     #[inline]
-    fn zero_index(&self) -> Self {
-        IxDyn::zeros(self.ndim())
-    }
-
-    #[inline]
-    fn zero_index_with_ndim(ndim: usize) -> Self {
+    fn zeros(ndim: usize) -> Self {
         IxDyn::zeros(ndim)
     }
 
