@@ -10,6 +10,7 @@
 //! Methods for two-dimensional arrays.
 use imp_prelude::*;
 
+/// # Methods For 2-D Arrays
 impl<A, S> ArrayBase<S, Ix2>
     where S: Data<Elem=A>,
 {
@@ -18,7 +19,7 @@ impl<A, S> ArrayBase<S, Ix2>
     /// **Panics** if `index` is out of bounds.
     pub fn row(&self, index: Ix) -> ArrayView1<A>
     {
-        self.subview(Axis(0), index)
+        self.index_axis(Axis(0), index)
     }
 
     /// Return a mutable array view of row `index`.
@@ -27,7 +28,7 @@ impl<A, S> ArrayBase<S, Ix2>
     pub fn row_mut(&mut self, index: Ix) -> ArrayViewMut1<A>
         where S: DataMut
     {
-        self.subview_mut(Axis(0), index)
+        self.index_axis_mut(Axis(0), index)
     }
 
     /// Return the number of rows (length of `Axis(0)`) in the two-dimensional array.
@@ -40,7 +41,7 @@ impl<A, S> ArrayBase<S, Ix2>
     /// **Panics** if `index` is out of bounds.
     pub fn column(&self, index: Ix) -> ArrayView1<A>
     {
-        self.subview(Axis(1), index)
+        self.index_axis(Axis(1), index)
     }
 
     /// Return a mutable array view of column `index`.
@@ -49,7 +50,7 @@ impl<A, S> ArrayBase<S, Ix2>
     pub fn column_mut(&mut self, index: Ix) -> ArrayViewMut1<A>
         where S: DataMut
     {
-        self.subview_mut(Axis(1), index)
+        self.index_axis_mut(Axis(1), index)
     }
 
     /// Return the number of columns (length of `Axis(1)`) in the two-dimensional array.
