@@ -23,8 +23,12 @@ use {
     Baseiter,
 };
 
-use iter;
-use iterators;
+use iter::{self, AxisIter, AxisIterMut};
+
+/// Returns `true` if the pointer is aligned.
+fn is_aligned<T>(ptr: *const T) -> bool {
+    (ptr as usize) % ::std::mem::align_of::<T>() == 0
+}
 
 /// Methods for read-only array views.
 impl<'a, A, D> ArrayView<'a, A, D>
@@ -58,27 +62,69 @@ impl<'a, A, D> ArrayView<'a, A, D>
         -> Result<Self, ShapeError>
         where Sh: Into<StrideShape<D>>,
     {
-        let shape = shape.into();
-        let dim = shape.dim;
-        let strides = shape.strides;
-        dimension::can_index_slice(xs, &dim, &strides).map(|_| {
-            unsafe {
-                Self::new_(xs.as_ptr(), dim, strides)
-            }
-        })
+        // eliminate the type parameter Sh as soon as possible
+        Self::from_shape_impl(shape.into(), xs)
     }
 
-    /// Create an `ArrayView<A, D>` from shape information and a
-    /// raw pointer to the elements.
+    fn from_shape_impl(shape: StrideShape<D>, xs: &'a [A]) -> Result<Self, ShapeError> {
+        let dim = shape.dim;
+        let strides = shape.strides;
+        if shape.custom {
+            dimension::can_index_slice(xs, &dim, &strides)?;
+        } else {
+            dimension::can_index_slice_not_custom::<A, _>(xs, &dim)?;
+        }
+        unsafe { Ok(Self::new_(xs.as_ptr(), dim, strides)) }
+    }
+
+    /// Create an `ArrayView<A, D>` from shape information and a raw pointer to
+    /// the elements.
     ///
-    /// Unsafe because caller is responsible for ensuring that the pointer is
-    /// valid, not mutably aliased and coherent with the dimension and stride information.
+    /// Unsafe because caller is responsible for ensuring all of the following:
+    ///
+    /// * The elements seen by moving `ptr` according to the shape and strides
+    ///   must live at least as long as `'a` and must not be not mutably
+    ///   aliased for the duration of `'a`.
+    ///
+    /// * `ptr` must be non-null and aligned, and it must be safe to
+    ///   [`.offset()`] `ptr` by zero.
+    ///
+    /// * It must be safe to [`.offset()`] the pointer repeatedly along all
+    ///   axes and calculate the `count`s for the `.offset()` calls without
+    ///   overflow, even if the array is empty or the elements are zero-sized.
+    ///
+    ///   In other words,
+    ///
+    ///   * All possible pointers generated by moving along all axes must be in
+    ///     bounds or one byte past the end of a single allocation with element
+    ///     type `A`. The only exceptions are if the array is empty or the element
+    ///     type is zero-sized. In these cases, `ptr` may be dangling, but it must
+    ///     still be safe to [`.offset()`] the pointer along the axes.
+    ///
+    ///   * The offset in units of bytes between the least address and greatest
+    ///     address by moving along all axes must not exceed `isize::MAX`. This
+    ///     constraint prevents the computed offset, in bytes, from overflowing
+    ///     `isize` regardless of the starting point due to past offsets.
+    ///
+    ///   * The offset in units of `A` between the least address and greatest
+    ///     address by moving along all axes must not exceed `isize::MAX`. This
+    ///     constraint prevents overflow when calculating the `count` parameter to
+    ///     [`.offset()`] regardless of the starting point due to past offsets.
+    ///
+    /// * The product of non-zero axis lengths must not exceed `isize::MAX`.
+    ///
+    /// [`.offset()`]: https://doc.rust-lang.org/stable/std/primitive.pointer.html#method.offset
     pub unsafe fn from_shape_ptr<Sh>(shape: Sh, ptr: *const A) -> Self
         where Sh: Into<StrideShape<D>>
     {
         let shape = shape.into();
         let dim = shape.dim;
         let strides = shape.strides;
+        if cfg!(debug_assertions) {
+            assert!(!ptr.is_null(), "The pointer must be non-null.");
+            assert!(is_aligned(ptr), "The pointer must be aligned.");
+            dimension::max_abs_offset_check_overflow::<A, _>(&dim, &strides).unwrap();
+        }
         ArrayView::new_(ptr, dim, strides)
     }
 
@@ -100,7 +146,7 @@ impl<'a, A, D> ArrayView<'a, A, D>
     /// Below, an illustration of `.split_at(Axis(2), 2)` on
     /// an array with shape 3 × 5 × 5.
     ///
-    /// <img src="https://bluss.github.io/ndarray/images/split_at.svg" width="300px" height="271px">
+    /// <img src="https://rust-ndarray.github.io/ndarray/images/split_at.svg" width="300px" height="271px">
     pub fn split_at(self, axis: Axis, index: Ix)
         -> (Self, Self)
     {
@@ -313,27 +359,69 @@ impl<'a, A, D> ArrayViewMut<'a, A, D>
         -> Result<Self, ShapeError>
         where Sh: Into<StrideShape<D>>,
     {
-        let shape = shape.into();
+        // eliminate the type parameter Sh as soon as possible
+        Self::from_shape_impl(shape.into(), xs)
+    }
+
+    fn from_shape_impl(shape: StrideShape<D>, xs: &'a mut [A]) -> Result<Self, ShapeError> {
         let dim = shape.dim;
         let strides = shape.strides;
-        dimension::can_index_slice(xs, &dim, &strides).map(|_| {
-            unsafe {
-                Self::new_(xs.as_mut_ptr(), dim, strides)
-            }
-        })
+        if shape.custom {
+            dimension::can_index_slice(xs, &dim, &strides)?;
+        } else {
+            dimension::can_index_slice_not_custom::<A, _>(xs, &dim)?;
+        }
+        unsafe { Ok(Self::new_(xs.as_mut_ptr(), dim, strides)) }
     }
 
     /// Create an `ArrayViewMut<A, D>` from shape information and a
     /// raw pointer to the elements.
     ///
-    /// Unsafe because caller is responsible for ensuring that the pointer is
-    /// valid, not aliased and coherent with the dimension and stride information.
+    /// Unsafe because caller is responsible for ensuring all of the following:
+    ///
+    /// * The elements seen by moving `ptr` according to the shape and strides
+    ///   must live at least as long as `'a` and must not be aliased for the
+    ///   duration of `'a`.
+    ///
+    /// * `ptr` must be non-null and aligned, and it must be safe to
+    ///   [`.offset()`] `ptr` by zero.
+    ///
+    /// * It must be safe to [`.offset()`] the pointer repeatedly along all
+    ///   axes and calculate the `count`s for the `.offset()` calls without
+    ///   overflow, even if the array is empty or the elements are zero-sized.
+    ///
+    ///   In other words,
+    ///
+    ///   * All possible pointers generated by moving along all axes must be in
+    ///     bounds or one byte past the end of a single allocation with element
+    ///     type `A`. The only exceptions are if the array is empty or the element
+    ///     type is zero-sized. In these cases, `ptr` may be dangling, but it must
+    ///     still be safe to [`.offset()`] the pointer along the axes.
+    ///
+    ///   * The offset in units of bytes between the least address and greatest
+    ///     address by moving along all axes must not exceed `isize::MAX`. This
+    ///     constraint prevents the computed offset, in bytes, from overflowing
+    ///     `isize` regardless of the starting point due to past offsets.
+    ///
+    ///   * The offset in units of `A` between the least address and greatest
+    ///     address by moving along all axes must not exceed `isize::MAX`. This
+    ///     constraint prevents overflow when calculating the `count` parameter to
+    ///     [`.offset()`] regardless of the starting point due to past offsets.
+    ///
+    /// * The product of non-zero axis lengths must not exceed `isize::MAX`.
+    ///
+    /// [`.offset()`]: https://doc.rust-lang.org/stable/std/primitive.pointer.html#method.offset
     pub unsafe fn from_shape_ptr<Sh>(shape: Sh, ptr: *mut A) -> Self
         where Sh: Into<StrideShape<D>>
     {
         let shape = shape.into();
         let dim = shape.dim;
         let strides = shape.strides;
+        if cfg!(debug_assertions) {
+            assert!(!ptr.is_null(), "The pointer must be non-null.");
+            assert!(is_aligned(ptr), "The pointer must be aligned.");
+            dimension::max_abs_offset_check_overflow::<A, _>(&dim, &strides).unwrap();
+        }
         ArrayViewMut::new_(ptr, dim, strides)
     }
 
@@ -469,7 +557,7 @@ impl<'a, A, D> ArrayView<'a, A, D>
     }
 
     #[inline]
-    pub(crate) fn into_base_iter(self) -> Baseiter<'a, A, D> {
+    pub(crate) fn into_base_iter(self) -> Baseiter<A, D> {
         unsafe {
             Baseiter::new(self.ptr, self.dim, self.strides)
         }
@@ -477,7 +565,7 @@ impl<'a, A, D> ArrayView<'a, A, D>
 
     #[inline]
     pub(crate) fn into_elements_base(self) -> ElementsBase<'a, A, D> {
-        ElementsBase { inner: self.into_base_iter() }
+        ElementsBase::new(self)
     }
 
     pub(crate) fn into_iter_(self) -> Iter<'a, A, D> {
@@ -490,7 +578,7 @@ impl<'a, A, D> ArrayView<'a, A, D>
     pub fn into_outer_iter(self) -> iter::AxisIter<'a, A, D::Smaller>
         where D: RemoveAxis,
     {
-        iterators::new_outer_iter(self)
+        AxisIter::new(self, Axis(0))
     }
 
 }
@@ -503,6 +591,11 @@ impl<'a, A, D> ArrayViewMut<'a, A, D>
     /// Unsafe because: `ptr` must be valid for the given dimension and strides.
     #[inline(always)]
     pub(crate) unsafe fn new_(ptr: *mut A, dim: D, strides: D) -> Self {
+        if cfg!(debug_assertions) {
+            assert!(!ptr.is_null(), "The pointer must be non-null.");
+            assert!(is_aligned(ptr), "The pointer must be aligned.");
+            dimension::max_abs_offset_check_overflow::<A, _>(&dim, &strides).unwrap();
+        }
         ArrayViewMut {
             data: ViewRepr::new(),
             ptr: ptr,
@@ -519,7 +612,7 @@ impl<'a, A, D> ArrayViewMut<'a, A, D>
     }
 
     #[inline]
-    pub(crate) fn into_base_iter(self) -> Baseiter<'a, A, D> {
+    pub(crate) fn into_base_iter(self) -> Baseiter<A, D> {
         unsafe {
             Baseiter::new(self.ptr, self.dim, self.strides)
         }
@@ -527,7 +620,7 @@ impl<'a, A, D> ArrayViewMut<'a, A, D>
 
     #[inline]
     pub(crate) fn into_elements_base(self) -> ElementsBaseMut<'a, A, D> {
-        ElementsBaseMut { inner: self.into_base_iter() }
+        ElementsBaseMut::new(self)
     }
 
     pub(crate) fn into_slice_(self) -> Result<&'a mut [A], Self> {
@@ -550,7 +643,7 @@ impl<'a, A, D> ArrayViewMut<'a, A, D>
     pub fn into_outer_iter(self) -> iter::AxisIterMut<'a, A, D::Smaller>
         where D: RemoveAxis,
     {
-        iterators::new_outer_iter_mut(self)
+        AxisIterMut::new(self, Axis(0))
     }
 }
 
