@@ -1,3 +1,4 @@
+use std::marker::PhantomData;
 
 use imp_prelude::*;
 use {NdProducer, Layout};
@@ -30,29 +31,30 @@ pub struct Lanes<'a, A: 'a, D> {
     inner_stride: Ixs,
 }
 
-
-pub fn new_lanes<A, D>(v: ArrayView<A, D>, axis: Axis)
-    -> Lanes<A, D::Smaller>
-    where D: Dimension
-{
-    let ndim = v.ndim();
-    let len;
-    let stride;
-    let iter_v;
-    if ndim == 0 {
-        len = 1;
-        stride = 1;
-        iter_v = v.try_remove_axis(Axis(0))
-    } else {
-        let i = axis.index();
-        len = v.dim[i];
-        stride = v.strides[i] as isize;
-        iter_v = v.try_remove_axis(axis)
-    }
-    Lanes {
-        inner_len: len,
-        inner_stride: stride,
-        base: iter_v,
+impl<'a, A, D: Dimension> Lanes<'a, A, D> {
+    pub(crate) fn new<Di>(v: ArrayView<'a, A, Di>, axis: Axis) -> Self
+    where
+        Di: Dimension<Smaller = D>,
+    {
+        let ndim = v.ndim();
+        let len;
+        let stride;
+        let iter_v;
+        if ndim == 0 {
+            len = 1;
+            stride = 1;
+            iter_v = v.try_remove_axis(Axis(0))
+        } else {
+            let i = axis.index();
+            len = v.dim[i];
+            stride = v.strides[i] as isize;
+            iter_v = v.try_remove_axis(axis)
+        }
+        Lanes {
+            inner_len: len,
+            inner_stride: stride,
+            base: iter_v,
+        }
     }
 }
 
@@ -84,6 +86,7 @@ impl<'a, A, D> IntoIterator for Lanes<'a, A, D>
             iter: self.base.into_base_iter(),
             inner_len: self.inner_len,
             inner_stride: self.inner_stride,
+            life: PhantomData,
         }
     }
 }
@@ -96,29 +99,30 @@ pub struct LanesMut<'a, A: 'a, D> {
     inner_stride: Ixs,
 }
 
-
-pub fn new_lanes_mut<A, D>(v: ArrayViewMut<A, D>, axis: Axis)
-    -> LanesMut<A, D::Smaller>
-    where D: Dimension
-{
-    let ndim = v.ndim();
-    let len;
-    let stride;
-    let iter_v;
-    if ndim == 0 {
-        len = 1;
-        stride = 1;
-        iter_v = v.try_remove_axis(Axis(0))
-    } else {
-        let i = axis.index();
-        len = v.dim[i];
-        stride = v.strides[i] as isize;
-        iter_v = v.try_remove_axis(axis)
-    }
-    LanesMut {
-        inner_len: len,
-        inner_stride: stride,
-        base: iter_v,
+impl<'a, A, D: Dimension> LanesMut<'a, A, D> {
+    pub(crate) fn new<Di>(v: ArrayViewMut<'a, A, Di>, axis: Axis) -> Self
+    where
+        Di: Dimension<Smaller = D>,
+    {
+        let ndim = v.ndim();
+        let len;
+        let stride;
+        let iter_v;
+        if ndim == 0 {
+            len = 1;
+            stride = 1;
+            iter_v = v.try_remove_axis(Axis(0))
+        } else {
+            let i = axis.index();
+            len = v.dim[i];
+            stride = v.strides[i] as isize;
+            iter_v = v.try_remove_axis(axis)
+        }
+        LanesMut {
+            inner_len: len,
+            inner_stride: stride,
+            base: iter_v,
+        }
     }
 }
 
@@ -132,6 +136,7 @@ impl<'a, A, D> IntoIterator for LanesMut<'a, A, D>
             iter: self.base.into_base_iter(),
             inner_len: self.inner_len,
             inner_stride: self.inner_stride,
+            life: PhantomData,
         }
     }
 }
