@@ -15,30 +15,31 @@ pub struct Windows<'a, A: 'a, D> {
     strides: D,
 }
 
-pub fn windows<A, D, E>(a: ArrayView<A, D>, window_size: E) -> Windows<A, D>
-	where D: Dimension,
-          E: IntoDimension<Dim=D>,
-{
-    let window = window_size.into_dimension();
-    ndassert!(a.ndim() == window.ndim(),
-        concat!("Window dimension {} does not match array dimension {} ",
-        "(with array of shape {:?})"),
-        window.ndim(), a.ndim(), a.shape());
-    let mut size = a.dim;
-    for (sz, &ws) in size.slice_mut().iter_mut().zip(window.slice())
+impl<'a, A, D: Dimension> Windows<'a, A, D> {
+    pub(crate) fn new<E>(a: ArrayView<'a, A, D>, window_size: E) -> Self
+    where
+        E: IntoDimension<Dim = D>,
     {
-        if ws == 0 { panic!("window-size must not be zero!"); }
-        // cannot use std::cmp::max(0, ..) since arithmetic underflow panics
-        *sz = if *sz < ws { 0 } else { *sz - ws + 1 };
-    }
+        let window = window_size.into_dimension();
+        ndassert!(a.ndim() == window.ndim(),
+            concat!("Window dimension {} does not match array dimension {} ",
+            "(with array of shape {:?})"),
+            window.ndim(), a.ndim(), a.shape());
+        let mut size = a.dim;
+        for (sz, &ws) in size.slice_mut().iter_mut().zip(window.slice()) {
+            assert_ne!(ws, 0, "window-size must not be zero!");
+            // cannot use std::cmp::max(0, ..) since arithmetic underflow panics
+            *sz = if *sz < ws { 0 } else { *sz - ws + 1 };
+        }
 
-    let window_strides = a.strides.clone();
+        let window_strides = a.strides.clone();
 
-    unsafe {
-        Windows {
-            base: ArrayView::from_shape_ptr(size.clone().strides(a.strides), a.ptr),
-            window: window,
-            strides: window_strides,
+        unsafe {
+            Windows {
+                base: ArrayView::from_shape_ptr(size.clone().strides(a.strides), a.ptr),
+                window: window,
+                strides: window_strides,
+            }
         }
     }
 }
