@@ -6,20 +6,16 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use std::ops::Add;
-use libnum::{self, Zero, Float};
+use std::ops::{Add, Div, Mul};
+use libnum::{self, One, Zero, Float};
 use itertools::free::enumerate;
 
 use imp_prelude::*;
 use numeric_util;
 
-use {
-    LinalgScalar,
-    FoldWhile,
-    Zip,
-};
+use {FoldWhile, Zip};
 
-/// Numerical methods for arrays.
+/// # Numerical Methods for Arrays
 impl<A, S, D> ArrayBase<S, D>
     where S: Data<Elem=A>,
           D: Dimension,
@@ -31,18 +27,18 @@ impl<A, S, D> ArrayBase<S, D>
     ///
     /// let a = arr2(&[[1., 2.],
     ///                [3., 4.]]);
-    /// assert_eq!(a.scalar_sum(), 10.);
+    /// assert_eq!(a.sum(), 10.);
     /// ```
-    pub fn scalar_sum(&self) -> A
+    pub fn sum(&self) -> A
         where A: Clone + Add<Output=A> + libnum::Zero,
     {
         if let Some(slc) = self.as_slice_memory_order() {
-            return numeric_util::unrolled_sum(slc);
+            return numeric_util::unrolled_fold(slc, A::zero, A::add);
         }
         let mut sum = A::zero();
         for row in self.inner_rows() {
             if let Some(slc) = row.as_slice() {
-                sum = sum + numeric_util::unrolled_sum(slc);
+                sum = sum + numeric_util::unrolled_fold(slc, A::zero, A::add);
             } else {
                 sum = sum + row.iter().fold(A::zero(), |acc, elt| acc + elt.clone());
             }
@@ -50,6 +46,43 @@ impl<A, S, D> ArrayBase<S, D>
         sum
     }
 
+    /// Return the sum of all elements in the array.
+    ///
+    /// *This method has been renamed to `.sum()` and will be deprecated in the
+    /// next version.*
+    // #[deprecated(note="renamed to `sum`", since="0.13")]
+    pub fn scalar_sum(&self) -> A
+        where A: Clone + Add<Output=A> + libnum::Zero,
+    {
+        self.sum()
+    }
+
+    /// Return the product of all elements in the array.
+    ///
+    /// ```
+    /// use ndarray::arr2;
+    ///
+    /// let a = arr2(&[[1., 2.],
+    ///                [3., 4.]]);
+    /// assert_eq!(a.product(), 24.);
+    /// ```
+    pub fn product(&self) -> A
+        where A: Clone + Mul<Output=A> + libnum::One,
+    {
+        if let Some(slc) = self.as_slice_memory_order() {
+            return numeric_util::unrolled_fold(slc, A::one, A::mul);
+        }
+        let mut sum = A::one();
+        for row in self.inner_rows() {
+            if let Some(slc) = row.as_slice() {
+                sum = sum * numeric_util::unrolled_fold(slc, A::one, A::mul);
+            } else {
+                sum = sum * row.iter().fold(A::one(), |acc, elt| acc * elt.clone());
+            }
+        }
+        sum
+    }
+
     /// Return sum along `axis`.
     ///
     /// ```
@@ -71,17 +104,17 @@ impl<A, S, D> ArrayBase<S, D>
               D: RemoveAxis,
     {
         let n = self.len_of(axis);
-        let mut res = self.subview(axis, 0).to_owned();
+        let mut res = Array::zeros(self.raw_dim().remove_axis(axis));
         let stride = self.strides()[axis.index()];
         if self.ndim() == 2 && stride == 1 {
             // contiguous along the axis we are summing
             let ax = axis.index();
             for (i, elt) in enumerate(&mut res) {
-                *elt = self.subview(Axis(1 - ax), i).scalar_sum();
+                *elt = self.index_axis(Axis(1 - ax), i).sum();
             }
         } else {
-            for i in 1..n {
-                let view = self.subview(axis, i);
+            for i in 0..n {
+                let view = self.index_axis(axis, i);
                 res = res + &view;
             }
         }
@@ -90,7 +123,8 @@ impl<A, S, D> ArrayBase<S, D>
 
     /// Return mean along `axis`.
     ///
-    /// **Panics** if `axis` is out of bounds.
+    /// **Panics** if `axis` is out of bounds or if the length of the axis is
+    /// zero and division by zero panics for type `A`.
     ///
     /// ```
     /// use ndarray::{aview1, arr2, Axis};
@@ -103,13 +137,13 @@ impl<A, S, D> ArrayBase<S, D>
     /// );
     /// ```
     pub fn mean_axis(&self, axis: Axis) -> Array<A, D::Smaller>
-        where A: LinalgScalar,
+        where A: Clone + Zero + One + Add<Output=A> + Div<Output=A>,
               D: RemoveAxis,
     {
         let n = self.len_of(axis);
         let sum = self.sum_axis(axis);
-        let mut cnt = A::one();
-        for _ in 1..n {
+        let mut cnt = A::zero();
+        for _ in 0..n {
             cnt = cnt + A::one();
         }
         sum / &aview0(&cnt)
@@ -175,10 +209,57 @@ impl<A, S, D> ArrayBase<S, D>
                     of the axis you are computing the variance for!")
         } else {
             let dof = count - ddof;
-            sum_sq.mapv(|s| s / dof)
+            sum_sq.mapv_into(|s| s / dof)
         }
     }
 
+    /// Return standard deviation along `axis`.
+    ///
+    /// The standard deviation is computed from the variance using
+    /// the [Welford one-pass algorithm](https://www.jstor.org/stable/1266577).
+    ///
+    /// The parameter `ddof` specifies the "delta degrees of freedom". For
+    /// example, to calculate the population standard deviation, use `ddof = 0`,
+    /// or to calculate the sample standard deviation, use `ddof = 1`.
+    ///
+    /// The standard deviation is defined as:
+    ///
+    /// ```text
+    ///                    1       n
+    /// stddev = sqrt ( ――――――――   ∑ (xᵢ - x̅)² )
+    ///                 n - ddof  i=1
+    /// ```
+    ///
+    /// where
+    ///
+    /// ```text
+    ///     1   n
+    /// x̅ = ―   ∑ xᵢ
+    ///     n  i=1
+    /// ```
+    ///
+    /// **Panics** if `ddof` is greater than or equal to the length of the
+    /// axis, if `axis` is out of bounds, or if the length of the axis is zero.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use ndarray::{aview1, arr2, Axis};
+    ///
+    /// let a = arr2(&[[1., 2.],
+    ///                [3., 4.],
+    ///                [5., 6.]]);
+    /// let stddev = a.std_axis(Axis(0), 1.);
+    /// assert_eq!(stddev, aview1(&[2., 2.]));
+    /// ```
+    pub fn std_axis(&self, axis: Axis, ddof: A) -> Array<A, D::Smaller>
+    where
+        A: Float,
+        D: RemoveAxis,
+    {
+        self.var_axis(axis, ddof).mapv_into(|x| x.sqrt())
+    }
+
     /// Return `true` if the arrays' elementwise differences are all within
     /// the given absolute tolerance, `false` otherwise.
     ///
