@@ -314,7 +314,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayView<'a, A, D>
 
     #[doc(hidden)]
     fn stride_of(&self, axis: Axis) -> isize {
-        self.strides()[axis.index()]
+        self.stride_of(axis)
     }
 
     #[inline(always)]
@@ -365,7 +365,7 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
 
     #[doc(hidden)]
     fn stride_of(&self, axis: Axis) -> isize {
-        self.strides()[axis.index()]
+        self.stride_of(axis)
     }
 
     #[inline(always)]
@@ -428,25 +428,23 @@ impl<'a, A, D: Dimension> NdProducer for ArrayViewMut<'a, A, D> {
 ///         *w += x + y * z;
 ///     });
 ///
-/// // Example 2: Create a new array `e` with one entry per row of `a`.
+/// // Example 2: Create a new array `totals` with one entry per row of `a`.
 /// //  Use Zip to traverse the rows of `a` and assign to the corresponding
-/// //  entry in `e` with the sum across each row.
-/// //  This is possible because the producer for `e` and the row producer
+/// //  entry in `totals` with the sum across each row.
+/// //  This is possible because the producer for `totals` and the row producer
 /// //  for `a` have the same shape and dimensionality.
 /// //  The rows producer yields one array view (`row`) per iteration.
 ///
 /// use ndarray::{Array1, Axis};
 ///
-/// let mut e = Array1::zeros(a.rows());
+/// let mut totals = Array1::zeros(a.rows());
 ///
-/// Zip::from(&mut e)
+/// Zip::from(&mut totals)
 ///     .and(a.genrows())
-///     .apply(|e, row| {
-///         *e = row.scalar_sum();
-///     });
+///     .apply(|totals, row| *totals = row.sum());
 ///
 /// // Check the result against the built in `.sum_axis()` along axis 1.
-/// assert_eq!(e, a.sum_axis(Axis(1)));
+/// assert_eq!(totals, a.sum_axis(Axis(1)));
 /// ```
 #[derive(Debug, Clone)]
 pub struct Zip<Parts, D> {
@@ -814,6 +812,7 @@ map_impl!{
 }
 
 /// Value controlling the execution of `.fold_while` on `Zip`.
+#[derive(Debug, Copy, Clone)]
 pub enum FoldWhile<T> {
     /// Continue folding with this value
     Continue(T),
