@@ -31,6 +31,16 @@ fn test_rc_into_owned() {
     assert!(!a.all_close(&c, 0.01));
 }
 
+#[test]
+fn test_arcarray_thread_safe() {
+    fn is_send<T: Send>(_t: &T) { }
+    fn is_sync<T: Sync>(_t: &T) { }
+    let a = Array2::from_elem((5, 5), 1.).into_shared();
+
+    is_send(&a);
+    is_sync(&a);
+}
+
 #[test]
 fn test_uninit() {
     unsafe {
@@ -132,6 +142,14 @@ fn deny_wraparound_from_vec() {
     assert!(six.is_err());
 }
 
+#[test]
+fn test_ones() {
+    let mut a = Array::<f32, _>::zeros((2, 3, 4));
+    a.fill(1.0);
+    let b = Array::<f32, _>::ones((2, 3, 4));
+    assert_eq!(a, b);
+}
+
 #[should_panic]
 #[test]
 fn deny_wraparound_zeros() {
