@@ -173,7 +173,7 @@ fn test_slice_array_fixed() {
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_inplace(info);
+    arr.view().slice_collapse(info);
 }
 
 #[test]
@@ -183,7 +183,7 @@ fn test_slice_dyninput_array_fixed() {
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_inplace(info.as_ref());
+    arr.view().slice_collapse(info.as_ref());
 }
 
 #[test]
@@ -197,7 +197,7 @@ fn test_slice_array_dyn() {
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_inplace(info);
+    arr.view().slice_collapse(info);
 }
 
 #[test]
@@ -211,7 +211,7 @@ fn test_slice_dyninput_array_dyn() {
     arr.slice(info);
     arr.slice_mut(info);
     arr.view().slice_move(info);
-    arr.view().slice_inplace(info.as_ref());
+    arr.view().slice_collapse(info.as_ref());
 }
 
 #[test]
@@ -225,7 +225,7 @@ fn test_slice_dyninput_vec_fixed() {
     arr.slice(info.as_ref());
     arr.slice_mut(info.as_ref());
     arr.view().slice_move(info.as_ref());
-    arr.view().slice_inplace(info.as_ref());
+    arr.view().slice_collapse(info.as_ref());
 }
 
 #[test]
@@ -239,7 +239,7 @@ fn test_slice_dyninput_vec_dyn() {
     arr.slice(info.as_ref());
     arr.slice_mut(info.as_ref());
     arr.view().slice_move(info.as_ref());
-    arr.view().slice_inplace(info.as_ref());
+    arr.view().slice_collapse(info.as_ref());
 }
 
 #[test]
@@ -253,7 +253,7 @@ fn test_slice_with_subview() {
     assert_eq!(vi.shape(), &[2, 2]);
     assert!(
         vi.iter()
-            .zip(arr.subview(Axis(1), 2).slice(s![1.., ..;2]).iter())
+            .zip(arr.index_axis(Axis(1), 2).slice(s![1.., ..;2]).iter())
             .all(|(a, b)| a == b)
     );
 
@@ -262,8 +262,8 @@ fn test_slice_with_subview() {
     assert!(
         vi.iter()
             .zip(
-                arr.subview(Axis(0), 1)
-                    .subview(Axis(0), 2)
+                arr.index_axis(Axis(0), 1)
+                    .index_axis(Axis(0), 2)
                     .slice(s![..;2])
                     .iter()
             )
@@ -276,7 +276,7 @@ fn test_slice_with_subview() {
 }
 
 #[test]
-fn test_slice_inplace_with_subview_inplace() {
+fn test_slice_collapse_with_indices() {
     let mut arr = RcArray::<usize, _>::zeros((3, 5, 4));
     for (i, elt) in arr.iter_mut().enumerate() {
         *elt = i;
@@ -284,7 +284,7 @@ fn test_slice_inplace_with_subview_inplace() {
 
     {
         let mut vi = arr.view();
-        vi.slice_inplace(s![1.., 2, ..;2]);
+        vi.slice_collapse(s![1.., 2, ..;2]);
         assert_eq!(vi.shape(), &[2, 1, 2]);
         assert!(
             vi.iter()
@@ -293,7 +293,7 @@ fn test_slice_inplace_with_subview_inplace() {
         );
 
         let mut vi = arr.view();
-        vi.slice_inplace(s![1, 2, ..;2]);
+        vi.slice_collapse(s![1, 2, ..;2]);
         assert_eq!(vi.shape(), &[1, 1, 2]);
         assert!(
             vi.iter()
@@ -302,7 +302,7 @@ fn test_slice_inplace_with_subview_inplace() {
         );
 
         let mut vi = arr.view();
-        vi.slice_inplace(s![1, 2, 3]);
+        vi.slice_collapse(s![1, 2, 3]);
         assert_eq!(vi.shape(), &[1, 1, 1]);
         assert_eq!(vi, Array3::from_elem((1, 1, 1), arr[(1, 2, 3)]));
     }
@@ -310,7 +310,7 @@ fn test_slice_inplace_with_subview_inplace() {
     // Do it to the RcArray itself
     let elem = arr[(1, 2, 3)];
     let mut vi = arr;
-    vi.slice_inplace(s![1, 2, 3]);
+    vi.slice_collapse(s![1, 2, 3]);
     assert_eq!(vi.shape(), &[1, 1, 1]);
     assert_eq!(vi, Array3::from_elem((1, 1, 1), elem));
 }
@@ -458,7 +458,7 @@ fn test_cow()
     assert_eq!(n[[0, 1]], 0);
     assert_eq!(n.get((0, 1)), Some(&0));
     let mut rev = mat.reshape(4);
-    rev.slice_inplace(s![..;-1]);
+    rev.slice_collapse(s![..;-1]);
     assert_eq!(rev[0], 4);
     assert_eq!(rev[1], 3);
     assert_eq!(rev[2], 2);
@@ -483,7 +483,7 @@ fn test_cow_shrink()
     // mutation shrinks the array and gives it different strides
     //
     let mut mat = RcArray::zeros((2, 3));
-    //mat.slice_inplace(s![.., ..;2]);
+    //mat.slice_collapse(s![.., ..;2]);
     mat[[0, 0]] = 1;
     let n = mat.clone();
     mat[[0, 1]] = 2;
@@ -498,7 +498,7 @@ fn test_cow_shrink()
     assert_eq!(n.get((0, 1)), Some(&0));
     // small has non-C strides this way
     let mut small = mat.reshape(6);
-    small.slice_inplace(s![4..;-1]);
+    small.slice_collapse(s![4..;-1]);
     assert_eq!(small[0], 6);
     assert_eq!(small[1], 5);
     let before = small.clone();
@@ -515,21 +515,21 @@ fn test_cow_shrink()
 fn test_sub()
 {
     let mat = RcArray::linspace(0., 15., 16).reshape((2, 4, 2));
-    let s1 = mat.subview(Axis(0), 0);
-    let s2 = mat.subview(Axis(0), 1);
+    let s1 = mat.index_axis(Axis(0), 0);
+    let s2 = mat.index_axis(Axis(0), 1);
     assert_eq!(s1.shape(), &[4, 2]);
     assert_eq!(s2.shape(), &[4, 2]);
     let n = RcArray::linspace(8., 15., 8).reshape((4,2));
     assert_eq!(n, s2);
     let m = RcArray::from_vec(vec![2., 3., 10., 11.]).reshape((2, 2));
-    assert_eq!(m, mat.subview(Axis(1), 1));
+    assert_eq!(m, mat.index_axis(Axis(1), 1));
 }
 
 #[should_panic]
 #[test]
 fn test_sub_oob_1() {
     let mat = RcArray::linspace(0., 15., 16).reshape((2, 4, 2));
-    mat.subview(Axis(0), 2);
+    mat.index_axis(Axis(0), 2);
 }
 
 
@@ -573,6 +573,97 @@ fn diag()
     assert_eq!(d.dim(), 1);
 }
 
+/// Check that the merged shape is correct.
+///
+/// Note that this does not check the strides in the "merged" case!
+#[test]
+fn merge_axes() {
+    macro_rules! assert_merged {
+        ($arr:expr, $slice:expr, $take:expr, $into:expr) => {
+            let mut v = $arr.slice($slice);
+            let merged_len = v.len_of(Axis($take)) * v.len_of(Axis($into));
+            assert!(v.merge_axes(Axis($take), Axis($into)));
+            assert_eq!(v.len_of(Axis($take)), if merged_len == 0 { 0 } else { 1 });
+            assert_eq!(v.len_of(Axis($into)), merged_len);
+        }
+    }
+    macro_rules! assert_not_merged {
+        ($arr:expr, $slice:expr, $take:expr, $into:expr) => {
+            let mut v = $arr.slice($slice);
+            let old_dim = v.raw_dim();
+            let old_strides = v.strides().to_owned();
+            assert!(!v.merge_axes(Axis($take), Axis($into)));
+            assert_eq!(v.raw_dim(), old_dim);
+            assert_eq!(v.strides(), &old_strides[..]);
+        }
+    }
+
+    let a = Array4::<u8>::zeros((3, 4, 5, 4));
+
+    assert_not_merged!(a, s![.., .., .., ..], 0, 0);
+    assert_merged!(a, s![.., .., .., ..], 0, 1);
+    assert_not_merged!(a, s![.., .., .., ..], 0, 2);
+    assert_not_merged!(a, s![.., .., .., ..], 0, 3);
+    assert_not_merged!(a, s![.., .., .., ..], 1, 0);
+    assert_not_merged!(a, s![.., .., .., ..], 1, 1);
+    assert_merged!(a, s![.., .., .., ..], 1, 2);
+    assert_not_merged!(a, s![.., .., .., ..], 1, 3);
+    assert_not_merged!(a, s![.., .., .., ..], 2, 1);
+    assert_not_merged!(a, s![.., .., .., ..], 2, 2);
+    assert_merged!(a, s![.., .., .., ..], 2, 3);
+    assert_not_merged!(a, s![.., .., .., ..], 3, 0);
+    assert_not_merged!(a, s![.., .., .., ..], 3, 1);
+    assert_not_merged!(a, s![.., .., .., ..], 3, 2);
+    assert_not_merged!(a, s![.., .., .., ..], 3, 3);
+
+    assert_merged!(a, s![.., .., .., ..;2], 0, 1);
+    assert_not_merged!(a, s![.., .., .., ..;2], 1, 0);
+    assert_merged!(a, s![.., .., .., ..;2], 1, 2);
+    assert_not_merged!(a, s![.., .., .., ..;2], 2, 1);
+    assert_merged!(a, s![.., .., .., ..;2], 2, 3);
+    assert_not_merged!(a, s![.., .., .., ..;2], 3, 2);
+
+    assert_merged!(a, s![.., .., .., ..3], 0, 1);
+    assert_not_merged!(a, s![.., .., .., ..3], 1, 0);
+    assert_merged!(a, s![.., .., .., ..3], 1, 2);
+    assert_not_merged!(a, s![.., .., .., ..3], 2, 1);
+    assert_not_merged!(a, s![.., .., .., ..3], 2, 3);
+
+    assert_merged!(a, s![.., .., ..;2, ..], 0, 1);
+    assert_not_merged!(a, s![.., .., ..;2, ..], 1, 0);
+    assert_not_merged!(a, s![.., .., ..;2, ..], 1, 2);
+    assert_not_merged!(a, s![.., .., ..;2, ..], 2, 3);
+
+    assert_merged!(a, s![.., ..;2, .., ..], 0, 1);
+    assert_not_merged!(a, s![.., ..;2, .., ..], 1, 0);
+    assert_not_merged!(a, s![.., ..;2, .., ..], 1, 2);
+    assert_merged!(a, s![.., ..;2, .., ..], 2, 3);
+    assert_not_merged!(a, s![.., ..;2, .., ..], 3, 2);
+
+    let a = Array4::<u8>::zeros((3, 1, 5, 1).f());
+    assert_merged!(a, s![.., .., ..;2, ..], 0, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 0, 3);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 0);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 2);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 3);
+    assert_merged!(a, s![.., .., ..;2, ..], 2, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 2, 3);
+    assert_merged!(a, s![.., .., ..;2, ..], 3, 0);
+    assert_merged!(a, s![.., .., ..;2, ..], 3, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 3, 2);
+    assert_merged!(a, s![.., .., ..;2, ..], 3, 3);
+
+    let a = Array4::<u8>::zeros((3, 0, 5, 1));
+    assert_merged!(a, s![.., .., ..;2, ..], 0, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 2, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 3, 1);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 0);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 2);
+    assert_merged!(a, s![.., .., ..;2, ..], 1, 3);
+}
+
 #[test]
 fn swapaxes()
 {
@@ -590,7 +681,7 @@ fn swapaxes()
 #[test]
 fn permuted_axes()
 {
-    let a = array![1].into_subview(Axis(0), 0);
+    let a = array![1].index_axis_move(Axis(0), 0);
     let permuted = a.view().permuted_axes([]);
     assert_eq!(a, permuted);
 
@@ -652,10 +743,14 @@ fn standard_layout()
     assert!(!a.is_standard_layout());
     a.swap_axes(0, 1);
     assert!(a.is_standard_layout());
-    let x1 = a.subview(Axis(0), 0);
+    let x1 = a.index_axis(Axis(0), 0);
     assert!(x1.is_standard_layout());
-    let x2 = a.subview(Axis(1), 0);
+    let x2 = a.index_axis(Axis(1), 0);
     assert!(!x2.is_standard_layout());
+    let x3 = ArrayView1::from_shape(1.strides(2), &[1]).unwrap();
+    assert!(x3.is_standard_layout());
+    let x4 = ArrayView2::from_shape((0, 2).strides((0, 1)), &[1, 2]).unwrap();
+    assert!(x4.is_standard_layout());
 }
 
 #[test]
@@ -678,7 +773,7 @@ fn assign()
     let mut a = arr2(&[[1, 2], [3, 4]]);
     {
         let mut v = a.view_mut();
-        v.slice_inplace(s![..1, ..]);
+        v.slice_collapse(s![..1, ..]);
         v.fill(0);
     }
     assert_eq!(a, arr2(&[[0, 0], [3, 4]]));
@@ -694,7 +789,23 @@ fn sum_mean()
     assert_eq!(a.mean_axis(Axis(1)), arr1(&[1.5, 3.5]));
     assert_eq!(a.sum_axis(Axis(1)).sum_axis(Axis(0)), arr0(10.));
     assert_eq!(a.view().mean_axis(Axis(1)), aview1(&[1.5, 3.5]));
-    assert_eq!(a.scalar_sum(), 10.);
+    assert_eq!(a.sum(), 10.);
+}
+
+#[test]
+fn sum_mean_empty() {
+    assert_eq!(Array3::<f32>::ones((2, 0, 3)).sum(), 0.);
+    assert_eq!(Array1::<f32>::ones(0).sum_axis(Axis(0)), arr0(0.));
+    assert_eq!(
+        Array3::<f32>::ones((2, 0, 3)).sum_axis(Axis(1)),
+        Array::zeros((2, 3)),
+    );
+    let a = Array1::<f32>::ones(0).mean_axis(Axis(0));
+    assert_eq!(a.shape(), &[]);
+    assert!(a[()].is_nan());
+    let a = Array3::<f32>::ones((2, 0, 3)).mean_axis(Axis(1));
+    assert_eq!(a.shape(), &[2, 3]);
+    a.mapv(|x| assert!(x.is_nan()));
 }
 
 #[test]
@@ -745,6 +856,53 @@ fn var_axis() {
     assert!(d.var_axis(Axis(0), 0.).all_close(&aview0(&1.8875), 1e-12));
 }
 
+#[test]
+fn std_axis() {
+    let a = array![
+       [
+            [ 0.22935481,  0.08030619,  0.60827517,  0.73684379],
+            [ 0.90339851,  0.82859436,  0.64020362,  0.2774583 ],
+            [ 0.44485313,  0.63316367,  0.11005111,  0.08656246]
+       ],
+       [
+           [ 0.28924665,  0.44082454,  0.59837736,  0.41014531],
+            [ 0.08382316,  0.43259439,  0.1428889 ,  0.44830176],
+            [ 0.51529756,  0.70111616,  0.20799415,  0.91851457]
+       ],
+    ];
+    assert!(a.std_axis(Axis(0), 1.5).all_close(
+        &aview2(&[
+            [ 0.05989184,  0.36051836,  0.00989781,  0.32669847],
+            [ 0.81957535,  0.39599997,  0.49731472,  0.17084346],
+            [ 0.07044443,  0.06795249,  0.09794304,  0.83195211],
+        ]),
+        1e-4,
+    ));
+    assert!(a.std_axis(Axis(1), 1.7).all_close(
+        &aview2(&[
+            [ 0.42698655,  0.48139215,  0.36874991,  0.41458724],
+            [ 0.26769097,  0.18941435,  0.30555015,  0.35118674],
+        ]),
+        1e-8,
+    ));
+    assert!(a.std_axis(Axis(2), 2.3).all_close(
+        &aview2(&[
+            [ 0.41117907,  0.37130425,  0.35332388],
+            [ 0.16905862,  0.25304841,  0.39978276],
+        ]),
+        1e-8,
+    ));
+
+    let b = array![[100000., 1., 0.01]];
+    assert!(b.std_axis(Axis(0), 0.).all_close(&aview1(&[0., 0., 0.]), 1e-12));
+    assert!(
+        b.std_axis(Axis(1), 0.).all_close(&aview1(&[47140.214021552769]), 1e-6),
+    );
+
+    let c = array![[], []];
+    assert_eq!(c.std_axis(Axis(0), 0.), aview1(&[]));
+}
+
 #[test]
 #[should_panic]
 fn var_axis_bad_dof() {
@@ -759,6 +917,20 @@ fn var_axis_empty_axis() {
     a.var_axis(Axis(1), 0.);
 }
 
+#[test]
+#[should_panic]
+fn std_axis_bad_dof() {
+    let a = array![1., 2., 3.];
+    a.std_axis(Axis(0), 4.);
+}
+
+#[test]
+#[should_panic]
+fn std_axis_empty_axis() {
+    let a = array![[], []];
+    a.std_axis(Axis(1), 0.);
+}
+
 #[test]
 fn iter_size_hint()
 {
@@ -810,7 +982,7 @@ fn zero_axes()
     println!("{:?}\n{:?}", b.shape(), b);
 
     // we can even get a subarray of b
-    let bsub = b.subview(Axis(0), 2);
+    let bsub = b.index_axis(Axis(0), 2);
     assert_eq!(bsub.dim(), 0);
 }
 
@@ -851,6 +1023,21 @@ fn as_slice_memory_order()
     assert!(a != b, "{:?} != {:?}", a, b);
 }
 
+#[test]
+fn array0_into_scalar() {
+    // With this kind of setup, the `Array`'s pointer is not the same as the
+    // underlying `Vec`'s pointer.
+    let a: Array0<i32> = array![4, 5, 6, 7].index_axis_move(Axis(0), 2);
+    assert_ne!(a.as_ptr(), a.into_raw_vec().as_ptr());
+    // `.into_scalar()` should still work correctly.
+    let a: Array0<i32> = array![4, 5, 6, 7].index_axis_move(Axis(0), 2);
+    assert_eq!(a.into_scalar(), 6);
+
+    // It should work for zero-size elements too.
+    let a: Array0<()> = array![(), (), (), ()].index_axis_move(Axis(0), 2);
+    assert_eq!(a.into_scalar(), ());
+}
+
 #[test]
 fn owned_array1() {
     let mut a = Array::from_vec(vec![1, 2, 3, 4]);
@@ -918,7 +1105,7 @@ fn owned_array_discontiguous_drop() {
         let v: Vec<_> = (0..12).map(|x| InsertOnDrop(set.clone(), Some(x))).collect();
         let mut a = Array::from_shape_vec((2, 6), v).unwrap();
         // discontiguous and non-zero offset
-        a.slice_inplace(s![.., 1..]);
+        a.slice_collapse(s![.., 1..]);
     }
     // each item was dropped exactly once
     itertools::assert_equal(set.borrow().iter().cloned(), 0..12);
@@ -1022,11 +1209,11 @@ fn from_vec_dim_stride_2d_7() {
 
 #[test]
 fn from_vec_dim_stride_2d_8() {
-    // strides must be strictly positive (nonzero)
+    // strides of length 1 axes can be zero
     let a = [1.];
     let d = (1, 1);
     let s = (0, 1);
-    assert_matches!(Array::from_shape_vec(d.strides(s), a.to_vec()), Err(_));
+    assert_matches!(Array::from_shape_vec(d.strides(s), a.to_vec()), Ok(_));
 }
 
 #[test]
@@ -1308,9 +1495,9 @@ fn insert_axis_f() {
 fn insert_axis_view() {
     let a = array![[[1, 2], [3, 4]], [[5, 6], [7, 8]], [[9, 10], [11, 12]]];
 
-    assert_eq!(a.subview(Axis(1), 0).insert_axis(Axis(0)), array![[[1, 2], [5, 6], [9, 10]]]);
-    assert_eq!(a.subview(Axis(1), 0).insert_axis(Axis(1)), array![[[1, 2]], [[5, 6]], [[9, 10]]]);
-    assert_eq!(a.subview(Axis(1), 0).insert_axis(Axis(2)), array![[[1], [2]], [[5], [6]], [[9], [10]]]);
+    assert_eq!(a.index_axis(Axis(1), 0).insert_axis(Axis(0)), array![[[1, 2], [5, 6], [9, 10]]]);
+    assert_eq!(a.index_axis(Axis(1), 0).insert_axis(Axis(1)), array![[[1, 2]], [[5, 6]], [[9, 10]]]);
+    assert_eq!(a.index_axis(Axis(1), 0).insert_axis(Axis(2)), array![[[1], [2]], [[5], [6]], [[9], [10]]]);
 }
 
 #[test]
@@ -1328,7 +1515,7 @@ fn char_array()
 {
     // test compilation & basics of non-numerical array
     let cc = RcArray::from_iter("alphabet".chars()).reshape((4, 2));
-    assert!(cc.subview(Axis(1), 0) == RcArray::from_iter("apae".chars()));
+    assert!(cc.index_axis(Axis(1), 0) == RcArray::from_iter("apae".chars()));
 }
 
 #[test]
@@ -1355,7 +1542,7 @@ fn scalar_ops() {
     let one = &zero + 1.;
     assert_eq!(0. * &one, zero);
     assert_eq!(&one * 0., zero);
-    assert_eq!((&one + &one).scalar_sum(), 8.);
+    assert_eq!((&one + &one).sum(), 8.);
     assert_eq!(&one / 2., 0.5 * &one);
     assert_eq!(&one % 1., zero);
 
@@ -1484,7 +1671,7 @@ fn to_owned_memory_order() {
 fn to_owned_neg_stride() {
     let mut c = arr2(&[[1, 2, 3],
                        [4, 5, 6]]);
-    c.slice_inplace(s![.., ..;-1]);
+    c.slice_collapse(s![.., ..;-1]);
     let co = c.to_owned();
     assert_eq!(c, co);
 }
@@ -1493,7 +1680,7 @@ fn to_owned_neg_stride() {
 fn discontiguous_owned_to_owned() {
     let mut c = arr2(&[[1, 2, 3],
                        [4, 5, 6]]);
-    c.slice_inplace(s![.., ..;2]);
+    c.slice_collapse(s![.., ..;2]);
 
     let co = c.to_owned();
     assert_eq!(c.strides(), &[3, 2]);
@@ -1646,10 +1833,10 @@ fn test_map_axis() {
                    [7, 8, 9],
                    [10,11,12]]);
 
-    let b = a.map_axis(Axis(0), |view| view.scalar_sum());
+    let b = a.map_axis(Axis(0), |view| view.sum());
     let answer1 = arr1(&[22, 26, 30]);
     assert_eq!(b, answer1);
-    let c = a.map_axis(Axis(1), |view| view.scalar_sum());
+    let c = a.map_axis(Axis(1), |view| view.sum());
     let answer2 = arr1(&[6, 15, 24, 33]);
     assert_eq!(c, answer2);
 }
@@ -1661,10 +1848,10 @@ fn test_to_vec() {
                        [7, 8, 9],
                        [10,11,12]]);
 
-    a.slice_inplace(s![..;-1, ..]);
+    a.slice_collapse(s![..;-1, ..]);
     assert_eq!(a.row(3).to_vec(), vec![1, 2, 3]);
     assert_eq!(a.column(2).to_vec(), vec![12, 9, 6, 3]);
-    a.slice_inplace(s![.., ..;-1]);
+    a.slice_collapse(s![.., ..;-1]);
     assert_eq!(a.row(3).to_vec(), vec![3, 2, 1]);
 }
 
@@ -1680,7 +1867,7 @@ fn test_array_clone_unalias() {
 #[test]
 fn test_array_clone_same_view() {
     let mut a = Array::from_iter(0..9).into_shape((3, 3)).unwrap();
-    a.slice_inplace(s![..;-1, ..;-1]);
+    a.slice_collapse(s![..;-1, ..;-1]);
     let b = a.clone();
     assert_eq!(a, b);
 }
