@@ -2,11 +2,9 @@
 extern crate ndarray;
 extern crate itertools;
 
-use ndarray::{Array0, Array2};
-use ndarray::RcArray;
+use ndarray::prelude::*;
 use ndarray::Ix;
 use ndarray::{
-    ArrayBase,
     Data,
     Dimension,
     aview1,
@@ -14,6 +12,7 @@ use ndarray::{
     arr3,
     Axis,
     indices,
+    Slice,
 };
 
 use itertools::assert_equal;
@@ -92,12 +91,12 @@ fn as_slice() {
     let a = a.reshape((2, 4));
     assert_slice_correct(&a);
 
-    assert!(a.view().subview(Axis(1), 0).as_slice().is_none());
+    assert!(a.view().index_axis(Axis(1), 0).as_slice().is_none());
 
     let v = a.view();
     assert_slice_correct(&v);
-    assert_slice_correct(&v.subview(Axis(0), 0));
-    assert_slice_correct(&v.subview(Axis(0), 1));
+    assert_slice_correct(&v.index_axis(Axis(0), 0));
+    assert_slice_correct(&v.index_axis(Axis(0), 1));
 
     assert!(v.slice(s![.., ..1]).as_slice().is_none());
     println!("{:?}", v.slice(s![..1;2, ..]));
@@ -180,12 +179,12 @@ fn outer_iter() {
     //   [8, 9],
     //    ...
     assert_equal(a.outer_iter(),
-                 vec![a.subview(Axis(0), 0), a.subview(Axis(0), 1)]);
+                 vec![a.index_axis(Axis(0), 0), a.index_axis(Axis(0), 1)]);
     let mut b = RcArray::zeros((2, 3, 2));
     b.swap_axes(0, 2);
     b.assign(&a);
     assert_equal(b.outer_iter(),
-                 vec![a.subview(Axis(0), 0), a.subview(Axis(0), 1)]);
+                 vec![a.index_axis(Axis(0), 0), a.index_axis(Axis(0), 1)]);
 
     let mut found_rows = Vec::new();
     for sub in b.outer_iter() {
@@ -210,7 +209,7 @@ fn outer_iter() {
     cv.assign(&a);
     assert_eq!(&a, &cv);
     assert_equal(cv.outer_iter(),
-                 vec![a.subview(Axis(0), 0), a.subview(Axis(0), 1)]);
+                 vec![a.index_axis(Axis(0), 0), a.index_axis(Axis(0), 1)]);
 
     let mut found_rows = Vec::new();
     for sub in cv.outer_iter() {
@@ -233,9 +232,9 @@ fn axis_iter() {
     //   [8, 9],
     //    ...
     assert_equal(a.axis_iter(Axis(1)),
-                 vec![a.subview(Axis(1), 0),
-                      a.subview(Axis(1), 1),
-                      a.subview(Axis(1), 2)]);
+                 vec![a.index_axis(Axis(1), 0),
+                      a.index_axis(Axis(1), 1),
+                      a.index_axis(Axis(1), 2)]);
 }
 
 #[test]
@@ -264,7 +263,7 @@ fn outer_iter_mut() {
     b.swap_axes(0, 2);
     b.assign(&a);
     assert_equal(b.outer_iter_mut(),
-                 vec![a.subview(Axis(0), 0), a.subview(Axis(0), 1)]);
+                 vec![a.index_axis(Axis(0), 0), a.index_axis(Axis(0), 1)]);
 
     let mut found_rows = Vec::new();
     for sub in b.outer_iter_mut() {
@@ -363,6 +362,31 @@ fn axis_chunks_iter_corner_cases() {
     assert_equal(it, vec![RcArray::zeros((1, 2)); 4]);
 }
 
+#[test]
+fn axis_chunks_iter_zero_stride() {
+
+    {
+        // stride 0 case
+        let b = Array::from_vec(vec![0f32; 0]).into_shape((5, 0, 3)).unwrap();
+        let shapes: Vec<_> = b.axis_chunks_iter(Axis(0), 2).map(|v| v.raw_dim()).collect();
+        assert_eq!(shapes, vec![Ix3(2, 0, 3), Ix3(2, 0, 3), Ix3(1, 0, 3)]);
+    }
+
+    {
+        // stride 0 case reverse
+        let b = Array::from_vec(vec![0f32; 0]).into_shape((5, 0, 3)).unwrap();
+        let shapes: Vec<_> = b.axis_chunks_iter(Axis(0), 2).rev().map(|v| v.raw_dim()).collect();
+        assert_eq!(shapes, vec![Ix3(1, 0, 3), Ix3(2, 0, 3), Ix3(2, 0, 3)]);
+    }
+
+    // From issue #542, ZST element
+    {
+        let a = Array::from_vec(vec![(); 3]);
+        let chunks: Vec<_> = a.axis_chunks_iter(Axis(0), 2).collect();
+        assert_eq!(chunks, vec![a.slice(s![0..2]), a.slice(s![2..])]);
+    }
+}
+
 #[test]
 fn axis_chunks_iter_mut() {
     let a = RcArray::from_iter(0..24);
@@ -492,9 +516,50 @@ fn test_fold() {
     a += 1;
     let mut iter = a.iter();
     iter.next();
-    assert_eq!(iter.fold(0, |acc, &x| acc + x), a.scalar_sum() - 1);
+    assert_eq!(iter.fold(0, |acc, &x| acc + x), a.sum() - 1);
 
     let mut a = Array0::<i32>::default(());
     a += 1;
     assert_eq!(a.iter().fold(0, |acc, &x| acc + x), 1);
 }
+
+#[test]
+fn test_rfold() {
+    {
+        let mut a = Array1::<i32>::default(256);
+        a += 1;
+        let mut iter = a.iter();
+        iter.next();
+        assert_eq!(iter.rfold(0, |acc, &x| acc + x), a.sum() - 1);
+    }
+
+    // Test strided arrays
+    {
+        let mut a = Array1::<i32>::default(256);
+        a.slice_axis_inplace(Axis(0), Slice::new(0, None, 2));
+        a += 1;
+        let mut iter = a.iter();
+        iter.next();
+        assert_eq!(iter.rfold(0, |acc, &x| acc + x), a.sum() - 1);
+    }
+
+    {
+        let mut a = Array1::<i32>::default(256);
+        a.slice_axis_inplace(Axis(0), Slice::new(0, None, -2));
+        a += 1;
+        let mut iter = a.iter();
+        iter.next();
+        assert_eq!(iter.rfold(0, |acc, &x| acc + x), a.sum() - 1);
+    }
+
+    // Test order
+    {
+        let mut a = Array1::from_iter(0..20);
+        a.slice_axis_inplace(Axis(0), Slice::new(0, None, 2));
+        let mut iter = a.iter();
+        iter.next();
+        let output = iter.rfold(Vec::new(),
+            |mut acc, elt| { acc.push(*elt); acc });
+        assert_eq!(Array1::from_vec(output), Array::from_iter((1..10).rev().map(|i| i * 2)));
+    }
+}
